<?php
// +----------------------------------------------------------------------+
// | ModernBill [TM] .:. Client Billing System                            |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                          |
// +----------------------------------------------------------------------+
// | This source file is subject to the ModernBill End User License       |
// | Agreement (EULA), that is bundled with this package in the file      |
// | LICENSE, and is available at through the world-wide-web at           |
// | http://www.modernbill.com/extranet/LICENSE.txt                       |
// | If you did not receive a copy of the ModernBill license and are      |
// | unable to obtain it through the world-wide-web, please send a note   |
// | to license@modernbill.com so we can email you a copy immediately.    |
// +----------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>               |
// | Support: http://www.modernsupport.com/modernbill/                    |
// +----------------------------------------------------------------------+
// | ModernGigabyte and ModernBill are trademarks of ModernGigabyte, LLC. |
// +----------------------------------------------------------------------+

require_once dirname(__FILE__)."/../lib-tk/shield.php";
require_once mb_path("/lib-action/commonaction.php");

class SourceAction extends CommonAction {
	/**
	 * string  default data source name for this action
	 */
	public $source;
	
	/**
	 * array  default data source params for this action
	 */
	public $sourceParams;
	
	/**
	 * string  template type in data source
	 */
	public $template;
	
	function SourceAction() {
		CommonAction::CommonAction();
		$this->stringRep = "SourceAction";
		
		$this->source = "";
		$this->sourceParams = array();
	}

	function execute() {
		parent::execute();
		return $this->hasExecutedSuccessfully();
	}

	/**
	 * get HTML options from a table
	 *
	 * @param  array  tabular data
	 * @param  string  option key column
	 * @param  string  option value column
	 * @return  array  HTML options
	 */
	function getOptionsFromTable($table, $keycol, $valcol) {
		$options = array();
		foreach($table as $row) {
			$options[$row[$keycol]] = $row[$valcol];
		}
		return $options;
	}

	/**
	 * get update IDs as returned 'add' and 'remove' keys. Andy's technique.
	 *
	 * @param  array  of selected IDs
	 * @param  array  of current IDs
	 * @return  array  associative array of 'add' and 'remove' with the add and remove IDs
	 */
	function getRelationUpdateIDs($selectedIDs, $currentIDs) {
		return array(
			'add'    => array_values(array_diff($selectedIDs, $currentIDs)),
			'remove' => array_values(array_diff($currentIDs, $selectedIDs)),
		);
	}

	/**
 	 * get parameters for setting relations
	 *
	 * @param  array  passed to getRelationUpdateIDs()
	 * @param  array  passed to getRelationUpdateIDs()
	 * @param  string  parent element postfix for setting relations as in 'productIDs' for 'productIDs' and 'removeProductIDs'
	 * @param  string  child element for setting relations
	 * @return  array  relation params
	 */
	function getRelationParams($selectedIDs, $currentIDs, $parent, $child) {
		$updateIDs = $this->getRelationUpdateIDs($selectedIDs, $currentIDs);
		$params = array();
		if($updateIDs['add']) {
			$params[$parent] =
				array($child => $updateIDs['add']);
		}
		if($updateIDs['remove']) {
			$params['remove' . ucfirst($parent)] =
				array($child => $updateIDs['remove']);
		}
		
		return $params;
	}

	/**
	 * gets IDs for joined relations in MBAPI results
	 *
	 * @param  array  MBAPI results at the point where the group is
	 * @param  array  column name for the ID of the relation
	 * @return  array  IDs
	 */
	function getRelationIDs($group, $IDcol) {
		$IDs = array();
		foreach($group as $item) {
			$itemID = $item[$IDcol][0];
			if (is_numeric($itemID)) $IDs[$itemID] = true;
		}
		$IDs = array_keys($IDs);
		debugForAction('Found relations', $IDs);
		return $IDs;
	}

	/**
	 * assign HTML options to smarty from a MBAPI query subject
	 *
	 * @param  string  smarty variable name
	 * @param  MBAPISubject  reference to an MBAPI subject with key and name attributes set
	 */
	function assignOptionsFromSubject($var, &$subject) {
		$this->smartyVars[$var] = $subject->getOptions();
	}

	/**
	 * assign HTML multiselect options to smarty using MBAPI query objects
	 *
	 * @param  string  smarty variable postfix; becomes `'selected' . ucfirst($varPostfix)` and `'selectable' . ucfirst($varPostfix)`
	 * @param  MBAPISubject  reference to an MBAPI subject with key and name attributes set, for retrieving the possible options
	 * @param  string  request arg
	 * @param  MBQuery  MBAPI query object with path attribute specifying where to get relations
	 * @param  string  relation ID column name
	 */
	function assignMultiOptionsFromSubject($varPostfix, &$subject, $requestArg, &$relQuery, $relID) {
		$allOptions = $subject->getOptions();
		
		$userVal = $this->getUserDataFor($requestArg);
		if($userVal !== NULL) {
			$currentIDs = (array) $userVal;
		} else {
			$relSubject =& $relQuery->getSubject($relID);
			$currentIDs = $relSubject->getKeys();
		}
		
		$selectedOptions = $selectableOptions = array();
		foreach($allOptions as $optionID => $optionName) {
			if(in_array($optionID, $currentIDs)) {
				$selectedOptions[$optionID] = $optionName;
			} else {
				$selectableOptions[$optionID] = $optionName;
			}
		}
		
		$this->smartyVars['selected' . ucfirst($varPostfix)] = $selectedOptions;
		$this->smartyVars['selectable' . ucfirst($varPostfix)] = $selectableOptions;
	}
	
	/**
	 * assign HTML multiselect options to smarty for a many to one relation, based on the values in another many to one relation.
	 *
	 * @param  string  variable namespace prefix
	 * @param  int  ID of the first "one" datasource
	 * @param  DataSource  of the selected "many"
	 * @param  DataSource  of the selected "one"
	 * @param  DataSource  of the selectable "many"
	 * @param  DataSource  of the selectable "one"
	 * @param  boolean  whether to sync the items to the variable; defaults to false
	 
	 $this->assignMultiOptionsForRelationItems(
	 	"", $productID,
		
		new DataSource("products"),
			new DataSource("variants"),
		new DataSource("productVariables"),
			new DataSource("productVariableValues"),
		1
	 );
	 
	 */
	function assignMultiOptionsForRelationItems($namespace, $id, &$mainD, &$itemD, &$optionNameD, &$optionValueD, $sync = false) {
		if($sync) $this->syncRelationItemsToVariable($id, $mainD, $itemD, $optionNameD, $optionValueD);
	
		// Get the option values (many-to-one) for the option name (one-to-one) in our main object.
		$optionValueD->mergeParams(array($optionValueD->getOneToOneColFor($optionNameD) => $mainD->getIDByID($optionNameD, $id)));
		$allOptions = $optionValueD->getOptions();
		
		// Limit the item query to only active items.
		$itemD->limitToActive();
		// Limit the item query to those related one-to-one to our main object.
		$itemD->limitByIDByID($mainD, $id);
		// Limit the item query to those related one-to-one to the values found in the first query.
		$itemD->limitByKeysOf($optionValueD);
		
		$userVal = $this->getUserDataFor($namespace . ucfirst($itemD->getSource()));
		if($userVal !== NULL) {
			$currentIDs = (array) $userVal;
		} else {
			// Get the IDs of the values related one-to-one to the existing items.
			$currentIDs = $itemD->getKeysFor($optionValueD);
		}
		
		$selectedOptions = $selectableOptions = array();
		foreach($allOptions as $optionID => $optionName) {
			if(in_array($optionID, $currentIDs)) {
				$selectedOptions[$optionID] = $optionName;
			} else {
				$selectableOptions[$optionID] = $optionName;
			}
		}
		
		$this->smartyVars[$namespace . "Selected" . ucfirst($itemD->getSource())] = $selectedOptions;
		$this->smartyVars[$namespace . "Selectable"  . ucfirst($itemD->getSource())] = $selectableOptions;
	}
	
	function syncRelationItemsToVariable($id, &$mainD, &$itemD, &$optionNameD, &$optionValueD) {
		
	}
	
	/**
	 * synchronize product variants for a product according to the product type
	 *
	 * @param  int  product ID
	 */
	function syncProductVariants($productID) {
		$products = new DataSource("products");
		$product = $products->getOneByID($productID);
		$type = $products->getType();
		
		$variants = new DataSource("variants");
		$variants->limitByIDByID($products, $productID);
		
		$notNeededVariants = array();
		if($type == "domain") {
			$tlds = new DataSource("TLDs");
			$tldIDs = $tlds->getKeys();
			$variantTldIDs = $variants->getKeysFor($tlds);
			
			$neededTLDs = array_diff($tldIDs, $variantTldIDs);
			$notNeededTLDs = array_diff($variantTldIDs, $tldIDs);
			
			$newVariants = array();
			foreach($neededTLDs as $tldID) {
				$newVariants[] = array(
					"productID" => $productID,
					"productVariantDisplayName" => "",
					"productVariantActive" => 0,
					"tldID" => $tldID,
				);
			}

			if($newVariants) foreach($newVariants as $newVariant) insertFor("variants", $newVariant);

			foreach($variants->getAllByID() as $variantID => $variant) {
				if(!in_array($variant["tldID"], $tldIDs)) $notNeededVariants[] = $variantID;
			}			
		} else {
			if($variableID = $product["productVariableID"]) {
				$variables = new DataSource("productVariables");
				$values = new DataSource("productVariableValues");
				$values->limitByIDByID($variables, $variableID);
				$valueIDs = $values->getKeys();
				$variantValueIDs = $variants->getKeysFor($values);
				
				//show($variants);
				
				//show("value IDs");
				//show($valueIDs);
				//show("variant value IDs");
				//show($variantValueIDs);
				
				$neededValues = array_diff($valueIDs, $variantValueIDs);
				$notNeededValues = array_diff($variantValueIDs, $valueIDs);
				
				//show("needed values");
				//show($neededValues);
				//show("not needed values");
				//show($notNeededValues);

				$newVariants = array();
				foreach($neededValues as $valueID) {
					$newVariants[] = array(
						"productID" => $productID,
						"productVariantDisplayName" => "TRANS_VARIANT_TEMPLATE",
						"productVariantActive" => 0,
						"productVariableValueID" => $valueID,
					);
				}

				if($newVariants) foreach($newVariants as $newVariant) insertFor("variants", $newVariant);

				foreach($variants->getAllByID() as $variantID => $variant) {
					if(!in_array($variant["productVariableValueID"], $valueIDs)) $notNeededVariants[] = $variantID;
				}			
			} else {
				$needNew = true;
				foreach($variants->getAllByID() as $variantID => $variant) {
					if($variant["productVariableValueID"] || $variant["tldID"]) $notNeededVariants[] = $variantID;
					else if(!$variant["productVariableValueID"] && !$variant["tldID"]) $needNew = false;
				}
				
				if($needNew) {
					$newVariant = array(
						"productID" => $productID,
						"productVariantDisplayName" => "TRANS_VARIANT_TEMPLATE",
						"productVariantActive" => 1,
					);
					insertFor("variants", $newVariant);
				}
			}
		}

		if($notNeededVariants) {
			$removeVariantQuery = new MBQuery(
				"SetProducts", "update",
				array(
					"removeProductVariantIDs" => array(
						"productVariantID" => $notNeededVariants,
					),
				)
			);
			$this->addErrorsFromResult($r =& $removeVariantQuery->getResult());
		}
	}

	function &getSubjectFromSource($d) {
		if($this->err($s =& $d->getSubject(), "Unable to get subject from data source:" . $d->getSource())) {
			$this->addErrorsFromDataSource($d);
			return false;
		}
		return $s;
	}

	function &getTableFromSource($d) {
		if($this->err($t =& $d->getTable(), "Unable to get table from data source:" . $d->getSource())) {
			$this->addErrorsFromDataSource($d);
			return false;
		}
		return $t;
	}

	function getOptionsFromSource($d) {
		if($this->err($o =& $d->getOptions(), "Unable to get HTML options from data source:" . $d->getSource())) {
			$this->addErrorsFromDataSource($d);
			return false;
		}
		return $o;
	}

	/**
	 * assign HTML mutliselect options to smarty for mass altering of a boolean column of a table
	 *
	 * @param  string  smarty variable postfix; becomes `'selected' . ucfirst($varPostfix)` and `'selectable' . ucfirst($varPostfix)`
	 * @param  string  column in table which represents activation
	 * @param  Table  of the data to display
	 */
	function assignMultiOptionsForActivationOfTable($varPostfix, $col, &$t) {
		$selectedOptions = $selectableOptions = array();
		$userVal = $this->getUserDataFor($varPostfix);
		if($userVal === Null) {
			$at =& $t->getAssociatedTablesBy($col);
			if(is_object($at[0])) {
				$selectableOptions = $at[0]->getOptions();
			} else {
				$selectableOptions = array();
			}
			if(is_object($at[1])) {
				$selectedOptions = $at[1]->getOptions();
			} else {
				$selectedOptions = array();
			}
		} else {
			$o = $t->getOptions();
			$cids = (array) $userVal;
			foreach($o as $id => $name) {
				if(in_array($id, $cids)) {
					$selectedOptions[$id] = $name;
				} else {
					$selectableOptions[$id] = $name;
				}
			}
		}
		
		$this->smartyVars['selected' . ucfirst($varPostfix)] = $selectedOptions;
		$this->smartyVars['selectable' . ucfirst($varPostfix)] = $selectableOptions;
		//echo '<pre>';
		//print_r($this->smartyVars);
		//echo '</pre>';	
	}

	/**
	 * assign HTML mutliselect options to smarty for mass altering of a boolean column of a data source
	 *
	 * @param  string  boolean associated column name
	 * @param  DataSource  which has the boolean association
	 * @param  DataSource  which contains the objects of association
	 * @return  boolean  success
	 */
	function assignForActivationOfJoinedSources($col, &$d, &$r) {
		$name = $r->getSource();
		if(!$dt =& $this->getTableFromSource($d)) return false;
		if(!$rt =& $this->getTableFromSource($r)) return false;
		
		$t =& $dt->getJoinedWith($rt);
		$t->setKey($dt->getKey());
		$t->setName($rt->getName());

		$this->assignMultiOptionsForActivationOfTable($name, $col, $t);
		
		//echo '<pre>';
		//print_r($t);
		//echo '</pre>';	
		return true;
	}

	/**
	 * get relation params from a relation subject
	 *
	 * @param  array  MBAPISubject from which to get relations
	 * @param  string  the parent element prefix passed to setRelations()
	 * @param  string  the child clement passed to setRelations()
	 * @param  string  the request arg of selected IDs
	 * @return  array  params for a set command for updating the relations
	 */
	function getRelationParamsFromSubject(&$subject, $parent, $child, $requestArg) {
		return $this->getRelationParams((array) $_REQUEST[$requestArg], $subject->getKeys(), $parent, $child);
	}

	/**
	 * add errors from MBAPIResult
	 *
	 * @param  MBAPIResult  from which errors are propagated to the action
	 * @param  boolean  whether the result succeeded
	 */
	function addErrorsFromResult(&$result) {
		$errors = $result->getErrors();
   		foreach((array)$errors as $title => $mesg) {
   			$this->addError($title, $mesg);
   			return false;
   		}
		return true;
	}

	/**
	 * get server data as a Table object
	 *
	 * @return  Table  table of server information
	 */
	function &getServerTable() {
		$servers = $this->getServerData();
		$serverTable = new Table($servers, "configGroupID", "moduleDisplayName");
		return $serverTable;
	}

	/**
	 * function to get the client side panel action data
	 */
	function getPanelCustomActionData($packageID, $serverGroupID=0, $configGroupID=0){
		global $CONF;
		$d = new DataSource("serverGroups", array("serverGroupID" => $serverGroupID, "getPanels" => 1));
		$s = $d->getSubject();
		$data = $s->getStructureFrom(array("panels", "panel", "configGroupTransKey"));
		$structure = (array)$s->getStructureFrom(array("serverGroupName"));
		$serverGroupName = current($structure);
		if($data) {
			$panel = current($data);
		} else {
			return false;
		}
		if($panel) {
			//show($panel);
			$panelFile = strtolower($panel);
			if(file_exists(mb_path("/lib-mbapi/include/modules/panel/$panelFile.php"))) {
				require_once mb_path("/lib-mbapi/include/modules/panel/$panelFile.php");
				//silly call because something is required, this aspect of panel needs reworking...
				$panelObject = new $panel(array("subCommand" => "panelDummy","serverGroupID" => $serverGroupID));

				if ($panelObject->capabilities[PANEL_DISPLAY_CUSTOM_CLIENT_ACTION_TEMPLATE] == 1) {
					$themeVariables = $this->getThemeVariables();
					$module_info = $GLOBALS["moduleInfo"][$panel];

					if ($module_info["customActionTemplateFile"]) {
						$tempFile = mb_path(
							$themeVariables["panelThemeDirectory"]
								. "/" . strtolower($panel) . "/"
								. $module_info["customActionTemplateFile"]
						);
						return array($tempFile, $serverGroupName);
					}
					$templateContent = $module_info["customActionTemplate"];
					if (! $templateContent) {
						return false;
					}
						
					$tempFile = $CONF["smarty"]["cacheDirectory"]."/panelTmplc_".$panel."_".$serverGroupID.".tpl";
					if (file_put_contents($tempFile, $templateContent) === false) {
						return false;
					}
						
					return array($tempFile, $serverGroupName);
				}
				return false;
			}
		}
		return false;
	}
	
	/**
	 * function to get the text's from the panel modules for client input's required
	 */
	function getClientInputDataForService($packageID, $serverGroupID=0, $configGroupID=0, $type="message"){
		global $CONF;
		$d = new DataSource("serverGroups", array("serverGroupID" => $serverGroupID, "getPanels" => 1));
		$s =& $d->getSubject();
		$data = $s->getStructureFrom(array("panels", "panel", "configGroupTransKey"));
		if($data) {
			$panel = current($data);
		} else {
			return false;
		}
		if($panel) {
			//show($panel);
			$panelFile = strtolower($panel);
			if(file_exists(mb_path("/lib-mbapi/include/modules/panel/$panelFile.php"))) {
				require_once mb_path("/lib-mbapi/include/modules/panel/$panelFile.php");
				//silly call because something is required, this aspect of panel needs reworking...
				$panelObject = new $panel(array("subCommand" => "panelDummy","serverGroupID" => $serverGroupID));
				switch ($type){
					case "message":
					default:
						$x = $panelObject->panelGetClientActionString($packageID, $serverGroupID, $configGroupID);
						if ($x){
							return array(
								"message" => $x,
								"packageID" => $packageID,
								"serverGroupID" => $serverGroupID,
								"configGroupID" => $configGroupID,
							);
						}
					break;
					case "template":
						$x = $panelObject->panelGetClientActionForm($packageID, $serverGroupID, $configGroupID);
						if ($x){
							return array(
								"template" => $x,
								"packageID" => $packageID,
								"serverGroupID" => $serverGroupID,
								"configGroupID" => $configGroupID,
							);
						}
					break;
					case "attributes":
						$x = $panelObject->panelGetClientActionAttributes($packageID, $serverGroupID, $configGroupID);
						if ($x){
							return array(
								"attributes" => $x,
								"packageID" => $packageID,
								"serverGroupID" => $serverGroupID,
								"configGroupID" => $configGroupID,
							);
						}
					break;
				}
			}
		}
		return false;
	}

	/**
	 * get attributes for server group ID
	 *
	 * @author  Andy Christ
	 * @param  int  server group ID
	 * @param  array|NULL optional product information or NULL
	 * @param  boolean  optional flag for getting HTML options of questions to attributes
	 * @return  array  server attributes
	 */
	function &getAttributesForServerGroupID($serverGroupID, $product = NULL, $asOptions = false, $allowCustom = false) {
		global $CONF;
		
		if(!$serverGroupID) {
			require_once mb_path("/lib-action/include/wizard/questions.php");
			
			$r =& Questions::unpackAttributesFor(
				array(
					"productID" => "",
					"orderFormID" => "",
					//"configGroupID" => "",
				)
			);
			if($asOptions) {
				$options = array();
				foreach((array)$r as $attrName => $attribute) {
					$options[$attrName] = $attribute["question"]->getQuestion();
				}
				return $options;
			} else {
				return $r;
			}
		}

		require_once mb_path("/lib-tk/toolkit.php");
		$d = new DataSource("serverGroups", array("serverGroupID" => $serverGroupID, "getPanels" => 1));
		$s =& $d->getSubject();
		$data = $s->getStructureFrom(array("panels", "panel", "configGroupTransKey"));
		if($data) {
			$panel = current($data);
		} else {
			return false;
		}

		if($panel) {
			//show($panel);
			$panelFile = strtolower($panel);

			// We need to skip this if the attributes screen is not the one being viewed.
			// As it is now, this will make the getProducts call out to the panel when the
			// details screen is being viewed
			// $this->[selectedButton] => attributes	Should make the call
			// $this->[selectedButton] => details		Should not make the call
			// -AC 07-05-06

			if(file_exists(mb_path("/lib-mbapi/include/modules/panel/$panelFile.php"))) {
				require_once mb_path("/lib-mbapi/include/modules/panel/$panelFile.php");
				//silly call because something is required, this aspect of panel needs reworking...
				$panelObject = new $panel(array("subCommand" => "panelDummy","serverGroupID" => $serverGroupID));


				if ($allowCustom && !(int)$product["productIsAddon"] && $panelObject->capabilities[PANEL_USE_CUSTOM_ATTRIBUTE_TEMPLATE]==1){
					$themeVariables = $this->getThemeVariables();
					$moduleInfo = $GLOBALS["moduleInfo"][$panel];
					$templateVariables = $panelObject->panelGetCustomTemplateVariables();
					$r = false;
					
					if ($moduleInfo["attributesTemplateFile"]) {
						$tempFile = mb_path(
							$themeVariables["panelThemeDirectory"]
								. "/" . strtolower($panel) . "/"
								. $moduleInfo["attributesTemplateFile"]
						);
					} else {
						$templateContent = $moduleInfo["attributesTemplate"];
						$tempFile = $CONF["smarty"]["cacheDirectory"]."/panelTmpl_".$panel."_".$serverGroupID.".tpl";
						if (file_put_contents($tempFile, $templateContent) === false) {
							return false;
						}
					}
					$r = array($tempFile, $templateVariables);
				} else {

					if($asOptions) {
						$r = $panelObject->getPanelDisplayOptions();
					} else {
						if($product["productIsAddon"] && $panelObject->capabilities[PANEL_ADDON_USAGE]) {
							$r = $panelObject->getPanelAddonDisplayItems();
						} else if($product["productType"] == getAPITypeID("products", "usage") && $panelObject->capabilities[PANEL_ADDON_USAGE]) {
							$r = $panelObject->getPanelAddonDisplayItems();
						} else {
							$r = $panelObject->getPanelDisplayItems();
						}
						
					}
	
					//$r = call_user_func(array($panel,"getPanelDisplayItems")); This line is changed to the 2 above
					// above line used because this doesn't parse
					//$r = $panel::getPanelDisplayItems();
					
					if($product !== NULL && (int)$product['productIsAddon'] && (int)$product["productAddonGroupID"]) {
						$attrName = $product['productAddonAttributeName'];
						$newR = array();
						$newR[$attrName] =& $r[$attrName];
						$r =& $newR;
					}
				}
				
			} else {
				dbg("WE HAD A SERVER GROUP TIED TO A PANEL THAT DOESN'T HAVE A PANEL FILE!");
				$r = false;
			}
		} else {
			dbg("WE COULDN'T FIGURE OUT THE PANEL FILE NAME!");
			$r = false;
		}

		return $r;
	}

	function mergeServerGroups($ids=0){
		if ($ids){
			$this->smartyVars["serverGroupList"] = optionsFor("serverGroups", array("serverGroupID" => $ids));
		} else {
			$this->smartyVars["serverGroupList"] = optionsFor("serverGroups", array());
		}
	}

}

