<?php

/**
 * Same as absolute_url but only protocol, host and port is returned.
 */
function absolute_url_server($server_props) {
    $host = 'localhost';
    if (!empty($server_props['HTTP_HOST'])) {
        list($host) = explode(':', $server_props['HTTP_HOST']);
    } elseif (!empty($server_props['SERVER_NAME'])) {
        list($host) = explode(':', $server_props['SERVER_NAME']);
    }

    if (isset($server_props['HTTPS']) && !strcasecmp($server_props['HTTPS'], 'on')) {
        $protocol = 'https';
    } else {
        $protocol = 'http';
    }
    $port = isset($server_props['SERVER_PORT']) ? $server_props['SERVER_PORT'] : 80;

    if ($protocol == 'http' && $port == 80) {
        unset($port);
    } else if ($protocol == 'https' && $port == 443) {
        unset($port);
    }

    return $protocol .'://'. $host . (isset($port) ? ':'. $port : '');
}


/**
 * Same as absolute_url but without url parameters
 */
function absolute_url_base($server_props, $url = null, $protocol = null, $port = null) {
    $parts = explode('?', absolute_url($server_props, $url, $protocol, $port));
    return $parts[0];
}

/**
 * Absolute URL
 *
 * This function returns the absolute URI for the partial URL passed.
 * The current scheme (HTTP/HTTPS), host server, port, current script
 * location are used if necessary to resolve any relative URLs.
 *
 * Offsets potentially created by PATH_INFO are taken care of to resolve
 * relative URLs to the current script.
 *
 * You can choose a new protocol while resolving the URI.  This is
 * particularly useful when redirecting a web browser using relative URIs
 * and to switch from HTTP to HTTPS, or vice-versa, at the same time.
 *
 * @author  Philippe Jausions <Philippe.Jausions@11abacus.com>
 * @author  Anonymous Parallels' coder. (code based on PECL's HTTP::absoluteURI function)
 * @return  string  The absolute URI.
 * @param   string  $url Absolute or relative URI the redirect should go to.
 * @param   string  $protocol Protocol to use when redirecting URIs.
 * @param   integer $port A new port number.
 */
function absolute_url($server_props, $url = null, $protocol = null, $port = null)
{
    // filter CR/LF
    $url = str_replace(array("\r", "\n"), ' ', $url);

    // Mess around with already absolute URIs
    if (preg_match('!^([a-z0-9]+)://!i', $url)) {
        if (empty($protocol) && empty($port)) {
            return $url;
        }
        if (!empty($protocol)) {
            $array = explode(':', $url, 2);
            $url = $protocol .':'. end($array);
        }
        if (!empty($port)) {
            $url = preg_replace('!^(([a-z0-9]+)://[^/:]+)(:[\d]+)?!i',
            '\1:'. $port, $url);
        }
        return $url;
    }

    $server = absolute_url_server($server_props);

    if (!strlen($url)) {
        $url = isset($server_props['REQUEST_URI']) ?
            $server_props['REQUEST_URI'] : $server_props['PHP_SELF'];
    }

    if ($url{0} == '/') {
        return $server . $url;
    }

    // Check for PATH_INFO
    if (isset($server_props['PATH_INFO'])
        && strlen($server_props['PATH_INFO'])
        && $server_props['PHP_SELF'] != $server_props['PATH_INFO']) {

        $path = dirname(substr($server_props['PHP_SELF'], 0, -strlen($server_props['PATH_INFO'])));
    } else {
        $path = dirname($server_props['PHP_SELF']);
    }

    if (substr($path = strtr($path, '\\', '/'), -1) != '/') {
        $path .= '/';
    }

    return $server . $path . $url;
}


/**
 * Append parameters to an existing URL string.
 *
 * @param string $url_string
 * @param array $parameters assoc array name --> value
 */
function add_url_parameters($url_string, $parameters) {
    if (strstr($url_string, '?')) {
        $delimiter = '&';
    } else {
        $delimiter = '?';
    }
    return $url_string . $delimiter . lib_PostData::urlencode($parameters);
}

?>